from os.path import dirname

from code_video import CodeScene


# Use the CodeScene to get the extra helper methods
class CommentedScene(CodeScene):
    def construct(self):
        example_dir = dirname(__file__)

        # Add a full screen background image to make the video a bit more interesting
        self.add_background(f"{example_dir}/resources/blackboard.jpg")

        # Display the code and animate comments as highlighted lines. For more control, use
        # highlight_line(s) directly.
        self.animate_code_comments(f"{example_dir}/commented.py", keep_comments=True, start_line=6, end_line=17)
        # end

        self.wait(5)



from __future__ import annotations

from textwrap import wrap
from typing import Dict
from typing import List
from typing import Optional

from manim import Arrow
from manim import BLACK
from manim import BLUE
from manim import DashedLine
from manim import DEFAULT_STROKE_WIDTH
from manim import DOWN
from manim import LEFT
from manim import Rectangle
from manim import RIGHT
from manim import Scene
from manim import ShowCreation
from manim import Text
from manim import UP
from manim import VGroup
from manim import WHITE
from manim.mobject.geometry import DEFAULT_DASH_LENGTH
from manim.mobject.geometry import Polygon
from numba import np


class Actor(VGroup):
    CONFIG = {
        "text_font": "Helvetica",
        "actor_fill_color": BLUE,
        "actor_text_color": BLACK,
    }

    def __init__(self, diagram: SequenceDiagram, title: str):
        super().__init__()
        self.diagram = diagram

        self.title = title

        tblock = VGroup()
        title = Text(title, font=self.CONFIG["text_font"], color=self.CONFIG["actor_text_color"])
        rect = (
            Rectangle(height=self._get_text_height(title) + 0.5, width=title.get_width() + 0.5)
            .round_corners(0.2)
            .set_fill(self.CONFIG["actor_fill_color"], 1)
        )
        title.move_to(rect)
        tblock.add(rect, title)
        self.block = tblock

        self.line = DashedLine(
            start=rect.get_edge_center(DOWN),
            end=[rect.get_center()[0], rect.get_bottom()[1], 0],
            stroke_style="dashed",
            dash_length=DEFAULT_DASH_LENGTH * 4,
            stroke_width=DEFAULT_STROKE_WIDTH / 2,
            positive_space_ratio=0.7,
        )
        self.bblock = tblock.copy()
        self.bblock.next_to(self.line, direction=DOWN, buff=0)
        self.add(tblock, self.line, self.bblock)

    def _get_text_height(self, text: Text) -> float:
        return max(Text("Ay", font=self.CONFIG["text_font"]).get_height(), text.get_height())

    def stretch(self, middle_height: float):
        self.remove(self.line, self.bblock)
        self.line = DashedLine(
            start=self.block.get_edge_center(DOWN),
            end=[self.block.get_center()[0], self.block.get_bottom()[1] - middle_height, 0],
            stroke_style="dashed",
            dash_length=DEFAULT_DASH_LENGTH * 4,
            stroke_width=DEFAULT_STROKE_WIDTH / 2,
            positive_space_ratio=0.7,
        )
        self.bblock = self.block.copy()
        self.bblock.next_to(self.line, direction=DOWN, buff=0)
        self.add(self.line, self.bblock)

    def text(self, value):
        self.diagram.interactions[-1].label = value
        return self

    def note(self, value: str):
        note_interaction = Note(self, value, RIGHT)
        interaction = self.diagram.interactions[-1]
        if not interaction.target:
            self.diagram.interactions.insert(-1, note_interaction)
        else:
            self.diagram.interactions.append(note_interaction)

    def to_self(self, value: str):
        note_interaction = SelfArrow(self, value)
        interaction = self.diagram.interactions[-1]
        if not interaction.target:
            self.diagram.interactions.insert(-1, note_interaction)
        else:
            self.diagram.interactions.append(note_interaction)

    def to_target(self, value: str, target: Actor):
        note_interaction = Interaction(source=self, label=value).finish(target)
        interaction = self.diagram.interactions[-1]
        if not interaction.target:
            self.diagram.interactions.insert(-1, note_interaction)
        else:
            self.diagram.interactions.append(note_interaction)

    def ret(self, value):
        interaction = self.diagram.interactions[-1]
        if not interaction.target:
            interaction = self.diagram.start_interaction(self)
        interaction.label = value
        return self.cur_interaction

    def __enter__(self):
        interaction = self.diagram.start_interaction(self)
        self.cur_interaction = interaction
        return self.cur_interaction

    def __exit__(self, exc_type, exc_val, exc_tb):
        interaction = self.diagram.start_interaction(self)
        self.cur_interaction = interaction
        return self.cur_interaction


class Interaction(VGroup):
    def __init__(self, source: Actor, label: str = "", target: Optional[Actor] = None, **kwargs):
        super().__init__(**kwargs)
        self.source = source
        self.target = target
        self.label = label

    def finish(self, target: Actor):
        self.target = target

        line = Arrow(start=[self.source.get_center()[0], 0, 0], end=[self.target.get_center()[0], 0, 0], buff=0)
        self.add(line)
        text = Text(self.label, font=self.source.CONFIG["text_font"]).scale(0.7).next_to(line, direction=UP, buff=0)
        self.add(text)
        return self

    def scale(self, scale_factor, **kwargs):
        super().scale(scale_factor, **kwargs)
        self.submobjects[0].align_to(
            self.source.get_center(),
            direction=LEFT if self.source.get_center()[0] < self.target.get_center()[0] else RIGHT,
        )
        self.submobjects[1].next_to(self.submobjects[0], direction=UP, buff=0)
        return self


class Note(Interaction):
    def __init__(self, target: Actor, label: str, direction: np.array):
        super().__init__(target)
        self.target = target
        self.label = "\n".join(wrap(label, 30))
        self.direction = direction

        block = VGroup()
        title = Text(self.label, font="Helvetica").scale(0.7)

        ear_size = title.get_width() * 0.08
        w = title.get_width() + 0.3 * 2
        h = title.get_height() + 0.3
        border = Polygon(
            (0, h, 0), (w - ear_size, h, 0), (w, h - ear_size, 0), (w, 0, 0), (0, 0, 0), (0, h, 0), color=WHITE
        )

        title.move_to(border)
        block.add(border, title)
        block.next_to(target.get_center(), direction)
        self.add(block)

    def scale(self, scale_factor, **kwargs):
        for obj in self.submobjects:
            obj.scale(scale_factor, **kwargs)
            obj.next_to(self.source.get_center(), direction=self.direction)
        return self

    def finish(self, target: Actor):
        raise NotImplementedError()


class SelfArrow(Interaction):
    def __init__(self, target: Actor, label: str):
        super().__init__(target)
        self.target = target
        self.label = "\n".join(wrap(label, 30))

        line_block = VGroup()

        spacing = 0.4
        distance = 0.8
        line = Polygon(
            [target.get_center()[0], spacing, 0],
            [target.get_center()[0] + distance, spacing, 0],
            [target.get_center()[0] + distance, -1 * spacing, 0],
            [target.get_center()[0] + distance / 2, -1 * spacing, 0],
            [target.get_center()[0] + distance, -1 * spacing, 0],
            [target.get_center()[0] + distance, spacing, 0],
            [target.get_center()[0], spacing, 0],
            color=WHITE,
        )

        arrow = Arrow(
            start=[target.get_center()[0] + distance, -1 * spacing, 0],
            end=[target.get_center()[0], -1 * spacing, 0],
            buff=0,
        )
        line_block.add(line, arrow)

        title = Text(self.label, font="Helvetica").scale(0.7)
        title.next_to(line_block)

        block = VGroup()
        block.add(line_block, title)
        block.next_to(target.get_center(), RIGHT)
        self.add(block)

    def scale(self, scale_factor, **kwargs):
        for obj in self.submobjects:
            obj.scale(scale_factor, **kwargs)
            obj.next_to(self.source.get_center(), direction=RIGHT, buff=0)
        return self

    def finish(self, target: Actor):
        raise NotImplementedError()


class SequenceDiagram(VGroup):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.actors: Dict[str, Actor] = {}
        self.interactions: List[Interaction] = []

    def add_objects(self, *object_names: str):
        for name in object_names:
            actor = Actor(self, name)
            if not self.actors:
                actor.to_edge(LEFT)
            else:
                actor.next_to(list(self.actors.values())[-1])
            actor.to_edge(UP)
            self.actors[name] = actor
            self.add(actor)

        start_x = list(self.actors.values())[0].get_x(LEFT)
        actor_width = max(max(actor.get_width() + 0.5 for actor in self.actors.values()), 5)
        for idx, actor in enumerate(self.actors.values()):
            left_x = start_x + actor_width * idx
            actor.set_x(left_x + (actor_width - actor.get_width()) / 2, LEFT)

        return self.actors.values()

    def start_interaction(self, actor: Actor):
        if self.interactions:
            last = self.interactions[-1]
            if last.source == actor:
                return last
            elif not last.target:
                last.finish(actor)
        interaction = Interaction(actor)
        self.interactions.append(interaction)
        return interaction

    def animate(self, scene: Scene):

        for actor in self.actors.values():
            actor.stretch(sum(item.get_height() + 0.5 for item in self.interactions))

        if scene.renderer.camera.frame_height < self.get_height() + 1.5:
            height_scale = scene.renderer.camera.frame_height / (self.get_height() + 1.5)
        else:
            height_scale = 1

        if scene.renderer.camera.frame_width < self.get_width() + 5:
            width_scale = scene.renderer.camera.frame_width / (self.get_width() + 5)
        else:
            width_scale = 1

        scale = min(1, height_scale, width_scale)

        self.scale(scale)
        self.to_edge(UP)
        self.to_edge(LEFT)
        start_y = self.get_edge_center(UP)[1] - 1.5 * scale

        scene.play(ShowCreation(self))

        last: Interaction = None
        for interaction in [item for item in self.interactions if item.target]:
            interaction.scale(scale)
            if not last:
                interaction.set_y(start_y, direction=UP)
            else:
                interaction.set_y(last.get_y(DOWN) - 0.5 * scale, direction=UP)

            scene.play(ShowCreation(interaction))
            last = interaction



from os.path import dirname

from manim import *

from code_video import CodeScene


class HighlightScene(CodeScene):
    def construct(self):
        example_dir = dirname(__file__)
        tex = self.create_code(f"{example_dir}/highlights.py")
        self.play(ShowCreation(tex))
        self.highlight_lines(
            tex,
            13,
            19,
            caption="Highlight code with a caption to give extra information. A wait is"
            " automatically added for a time based on the length of the caption",
        )
        self.highlight_line(tex, 21, caption="Reset highlighting and positioning")
        self.highlight_none(tex)
        self.wait(5)



from typing import List
from typing import Optional
from typing import Tuple

from pygments.lexers import get_lexer_for_filename


class Comment:
    def __init__(self):
        self.lines = []
        self.start = 1
        self.end = 1

    def append(self, other):
        self.lines.append(other)

    def __bool__(self):
        return bool(self.lines)

    @property
    def caption(self):
        return "\n".join(self.lines)


def parse(
    path: str, keep_comments: bool, start_line: int, end_line: Optional[int]
) -> Tuple[List[str], List[Comment]]:
    code: List[str] = []
    comments: List[Comment] = []
    lexer = get_lexer_for_filename(path)
    comment_marker = "#"
    if lexer.name in ("JavaScript", "Java", "C++"):
        comment_marker = "//"

    with open(path, "r") as f:
        comment = Comment()
        lines = f.readlines()[start_line - 1 :]

    if end_line and start_line - end_line < len(lines):
        lines = lines[: end_line - start_line + 1]
    for line in lines:
        stripped = line.strip()
        if stripped.startswith(f"{comment_marker} end"):
            last_comment = comments[-1]
            last_comment.end = len(code)
            if not keep_comments:
                continue
        elif stripped.startswith(comment_marker):
            comment.append(stripped[2:].strip())
            if not keep_comments:
                continue
        elif comment:
            comment.start = len(code) + 1
            comment.end = comment.start
            comments.append(comment)
            comment = Comment()
        code.append(line)

    if not code[-1].strip():
        code = code[:-1]
    return code, comments



import argparse
import sys


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--slides",
        action="store_const",
        const=True,
        help="Automatically open the videos as fullscreen slides once its done",
    )
    args, extra = parser.parse_known_args(sys.argv[1:])
    sys.argv = [sys.argv[0].replace("codevidgen", "manim")] + extra

    from manim.__main__ import main as manim_main
    from manim import config
    config["show_slides"] = args.slides
    config["slide_stops"] = []
    manim_main()
    if args.slides:
        from code_video.player import VideoPlayer
        partial_files = config["slide_videos"]
        pauses = config["slide_stops"]
        movie_file_path = config["movie_file_path"]
        player = VideoPlayer(clip_file_pattern=movie_file_path[:-4] + "-{index}.mp4")
        clip = []

        for idx, partial_file in enumerate([f for f in partial_files if f is not None]):
            clip.append(partial_file)
            if idx in pauses:
                if clip:
                    player.add_movies(*clip)
                    clip.clear()

        if clip:
            player.add_movies(*clip)
        player.play()


if __name__ == '__main__':
    main()


from os.path import dirname

from code_video import CodeScene
from code_video import SequenceDiagram


class SequenceDiagramsScene(CodeScene):
    def construct(self):
        example_dir = dirname(__file__)
        self.add_background(f"{example_dir}/resources/blackboard.jpg")
        diagram = SequenceDiagram()
        browser, web, app = diagram.add_objects("Browser", "Web", "App")
        with browser:
            with web.text("Make a request"):
                web.to_target("Do a quick thing", app)
                with app.text("Retrieve a json object"):
                    app.to_self("Calls itself")
                    app.note("Do lots and lots and lots of thinking")
                    app.ret("Value from db")
                web.ret("HTML response")

        diagram.animate(self)

        self.wait(5)



import os
import sys
from dataclasses import dataclass
from tempfile import NamedTemporaryFile
from typing import List
from typing import Optional

import ffmpeg
import pyglet
from pyglet import gl
from pyglet.media.codecs.ffmpeg import FFmpegSource
from pyglet.window import key


@dataclass
class Clip:
    file: str
    source: FFmpegSource


class VideoPlayer:
    def __init__(self, clip_file_pattern: str):
        self._window = pyglet.window.Window(fullscreen=True)
        self._window.event(self.on_draw)
        self._window.event(self.on_key_press)
        self._player = pyglet.media.Player()
        self._clips: List[Clip] = []
        self._clip_file_pattern = clip_file_pattern
        self.video_y: Optional[int] = None

        gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_MAG_FILTER, gl.GL_NEAREST)
        gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_MIN_FILTER, gl.GL_NEAREST)

    def add_movies(self, *movie_paths):
        clip_file_name = self._clip_file_pattern.format(index=len(self._clips))
        # noinspection PyTypeChecker
        with NamedTemporaryFile(mode="w") as file:
            paths = "\n-  ".join(movie_paths)
            print(f"Concat movie files into {clip_file_name}: \n-{paths}")
            for path in movie_paths:
                line = f"file '{os.path.abspath(path)}'\n"
                file.write(line)
            file.flush()
            (
                ffmpeg.input(file.name, format="concat", safe=0)
                .output(clip_file_name, c="copy")
                .run(overwrite_output=True)
            )

        print(f"Created {clip_file_name}")

        clip: FFmpegSource = pyglet.media.load(clip_file_name)
        self._clips.append(Clip(source=clip, file=clip_file_name))

        if not self.video_y:
            ratio_y = self._window.height / clip.video_format.height
            ratio_x = self._window.width / clip.video_format.width
            ratio = min(ratio_x, ratio_y)
            gl.glScalef(ratio, ratio, ratio)

            if ratio_x == ratio:
                self.video_y = ((self._window.height - clip.video_format.height * ratio) / ratio) / 2
            else:
                self.video_y = 0

        return self

    def play(self):
        clip = self._clips.pop(0)
        self._player.queue(clip.source)
        self._player.play()
        pyglet.app.run()
        return [video.file for video in self._clips]

    def on_draw(self):
        if self._player.source and self._player.source.video_format:
            self._player.texture.blit(0, self.video_y)

        if not self._player.playing:
            print("Paused")

    def on_key_press(self, symbol, modifiers):
        if symbol == key.Q:
            print("Q pressed!")
            self._window.close()

        if symbol in (key.SPACE, key.PAGEDOWN, key.RIGHT):
            if not self._clips:
                self._window.close()
                return
            print("Next video")
            self._player.pause()
            self.play()


if __name__ == "__main__":
    VideoPlayer("clip-{index}.mp4").add_movies(*sys.argv[1:-1]).add_movies(*sys.argv[-1:]).play()



from tempfile import NamedTemporaryFile

import librosa
from pydub import AudioSegment


class BackgroundMusic:
    def __init__(self, file: str):
        self.file = file
        x, sr = librosa.load(file)
        _, self.beat_times = librosa.beat.beat_track(x, sr=sr, start_bpm=60, units="time")
        self.beat_times = [0] + self.beat_times.tolist()
        self.off_beat_times = []
        for idx, time in enumerate(self.beat_times[::2]):
            if len(self.beat_times) >= idx:
                self.off_beat_times.append((self.beat_times[idx] + self.beat_times[idx + 1]) / 2)
        self.measure_times = self.off_beat_times[::2]

    def next_beat(self, time):
        for item in self.beat_times:
            if item >= time:
                return time + item - time
        raise ValueError("No more music")

    def next_measure(self, time):
        for item in self.measure_times:
            if item >= time:
                return item
        return time


def fit_audio(file: str, length: float) -> str:
    extension = file.split(".")[-1]
    segment = AudioSegment.from_file(file, extension)[: length * 1000].fade_out(1000)
    tmp = NamedTemporaryFile(suffix=f".{extension}", delete=False)
    segment.export(tmp.name, format=extension)
    return tmp.name



from code_video import CodeScene


class MyScene(CodeScene):
    def construct(self):
        # This does the actual code display and animation
        self.animate_code_comments("simple.py")

        # Wait 5 seconds before finishing
        self.wait(5)



import os
from tempfile import NamedTemporaryFile
from textwrap import wrap
from typing import Optional
from typing import Union

from manim import *

from code_video import comment_parser
from code_video.music import BackgroundMusic
from code_video.music import fit_audio


class CodeScene(MovingCameraScene):
    CONFIG = {
        "code_font": "Ubuntu Mono",
        "text_font": "Helvetica",
        "code_theme": "fruity",
    }

    def __init__(
        self,
        *args,
        **kwargs,
    ):
        super().__init__(*args, **kwargs)
        self.caption = None
        self.col_width = self.camera_frame.get_width() / 3
        self.music: Optional[BackgroundMusic] = None
        self.pauses = []

    def add_background_music(self, path: str):
        self.music = BackgroundMusic(path)

    def tear_down(self):
        super().tear_down()
        if self.music:
            self.time = 0
            file = fit_audio(self.music.file, self.renderer.time + 2)
            self.add_sound(file)
            os.remove(file)

        if self.pauses:
            config["slide_videos"] = self.renderer.file_writer.partial_movie_files[:]
            config["slide_stops"].extend(self.pauses)
            config["movie_file_path"] = self.renderer.file_writer.movie_file_path

    def wait(self, duration=DEFAULT_WAIT_TIME, stop_condition=None):
        if config.get("show_slides"):
            print("In slide mode, skipping wait")
            self.pauses.append(len(self.renderer.file_writer.partial_movie_files) - 1)
        else:
            super().wait(duration, stop_condition)

    def wait_until_beat(self, wait_time: Union[float, int]):
        if self.music:
            adjusted_delay = self.music.next_beat(self.renderer.time + wait_time) - self.renderer.time
            self.wait(adjusted_delay)
        else:
            self.wait(wait_time)

    def wait_until_measure(self, wait_time: Union[float, int], post: Union[float, int] = 0):
        if self.music:
            adjusted_delay = self.music.next_measure(self.renderer.time + wait_time) - self.renderer.time
            adjusted_delay += post
            self.wait(adjusted_delay)

        else:
            self.wait(wait_time)

    def add_background(self, path: str) -> ImageMobject:
        background = ImageMobject(path, height=self.camera_frame.get_height())
        background.stretch_to_fit_width(self.camera_frame.get_width())
        self.add(background)
        return background

    def animate_code_comments(
        self,
        path: str,
        title: str = None,
        keep_comments: bool = False,
        start_line: int = 1,
        end_line: Optional[int] = None,
        reset_at_end: bool = True,
    ) -> Code:

        parent = None
        if title:
            title = PangoText(title, font=self.CONFIG["text_font"]).to_edge(edge=UP)
            self.add(title)
            code_group = VGroup().next_to(title, direction=DOWN)
            self.add(code_group)
            parent = code_group

        code, comments = comment_parser.parse(
            path, keep_comments=keep_comments, start_line=start_line, end_line=end_line
        )

        with NamedTemporaryFile(suffix=f".{path.split('.')[-1]}") as f:
            f.writelines([line.encode() for line in code])
            f.flush()
            tex = self.create_code(f.name, line_no_from=start_line)
            if parent:
                parent.add(tex)
        self.play(ShowCreation(tex))
        self.wait()

        for comment in comments:
            self.highlight_lines(tex, comment.start, comment.end, comment.caption)

        if reset_at_end:
            self.highlight_none(tex)
        return tex

    def highlight_lines(self, tex: Code, start: int = 1, end: int = -1, caption: Optional[str] = None):
        if end == -1:
            end = len(tex.line_numbers) + 1

        def in_range(number: int):
            return start <= number <= end

        pre_actions = []
        actions = []
        post_actions = []

        if caption:
            caption = "\n".join(wrap(caption, 25))
            if self.caption:
                pre_actions.append(FadeOut(self.caption))
            else:
                self.play(ApplyMethod(tex.to_edge))

            self.caption = PangoText(
                caption, font=self.CONFIG["text_font"], size=self.col_width / 10 * 0.9
            ).add_background_rectangle(buff=MED_SMALL_BUFF)
            self.caption.next_to(tex, RIGHT)
            self.caption.align_to(tex.line_numbers[start - 1], UP)
            actions.append(FadeIn(self.caption))

        elif self.caption:
            actions.append(FadeOut(self.caption))
            post_actions += [ApplyMethod(tex.center)]
            self.caption = None

        # highlight code lines
        actions += [
            ApplyMethod(
                tex.code[line_no].set_opacity,
                1 if in_range(line_no + 1) else 0.3,
            )
            for line_no in range(len(tex.code))
        ]

        # highlight line numbers
        actions += [
            ApplyMethod(
                tex.line_numbers[line_no].set_opacity,
                1 if in_range(line_no + 1) else 0.3,
            )
            for line_no in range(len(tex.code))
        ]

        if pre_actions:
            self.play(*pre_actions)

        if actions:
            self.play(*actions)

        if caption:
            wait_time = len(caption) / (200 * 5 / 60)
            self.wait_until_measure(wait_time, -1.5)

        if post_actions:
            self.play(*post_actions)

    def highlight_line(self, tex: Code, number: int = -1, caption: Optional[str] = None):
        return self.highlight_lines(tex, number, number, caption=caption)

    def highlight_none(self, tex: Code):
        return self.highlight_lines(tex, 1, len(tex.code) + 1, caption=None)

    def create_code(self, path: str, **kwargs) -> Code:
        tex = Code(path, font=self.CONFIG["code_font"], style=self.CONFIG["code_theme"], **kwargs)
        x_scale = (self.col_width * 2) / tex.get_width()
        y_scale = self.camera_frame.get_height() * 0.95 / tex.get_height()
        tex.scale(min(x_scale, y_scale))
        return tex



from os.path import dirname

from manim import DOWN
from manim import FadeIn
from manim import FadeOut
from manim import LARGE_BUFF
from manim import LEFT
from manim import linear
from manim import MED_LARGE_BUFF
from manim import PangoText
from manim import ShowCreation

from code_video import CodeScene

example_dir = dirname(__file__)


def title_scene(scene):
    scene.add_background(f"{example_dir}/resources/blackboard.jpg")
    title = PangoText("How to use Code Video Generator", font="Helvetica")
    scene.play(ShowCreation(title))
    scene.play(
        FadeIn(
            PangoText("A code walkthrough", font="Helvetica")
            .scale(0.6)
            .next_to(title, direction=DOWN, buff=LARGE_BUFF)
        )
    )

    scene.wait(3)
    scene.clear()


def overview(scene):
    title = PangoText(
        """
    Manim is a Python library used to generate videos,
    and Code Video Generator provides a base scene that makes it easy
    to generate code walkthrough videos
    ... in fact, it is what was used to generate this video!
    """,
        font="Helvetica",
        line_spacing=0.5,
    ).scale(0.7)
    scene.play(ShowCreation(title, run_time=10, rate_func=linear))
    scene.wait(3)
    sub = (
        PangoText(
            """
        Here is an example:
        """,
            font="Helvetica",
        )
        .scale(0.7)
        .next_to(title, direction=DOWN, buff=MED_LARGE_BUFF, aligned_edge=LEFT)
    )
    scene.play(ShowCreation(sub))
    scene.wait(2)
    scene.clear()


def demo_commenting(scene: CodeScene):
    scene.add_background(f"{example_dir}/resources/blackboard.jpg")

    code = scene.animate_code_comments(
        title="examples/commented.py",
        path=f"{example_dir}/commented.py",
        keep_comments=True,
        start_line=6,
        end_line=19,
        reset_at_end=False,
    )

    scene.highlight_line(
        code,
        number=6,
        caption="These caption callouts are "
        "automatically generated from comments when "
        "using animate_code_comments()",
    )
    scene.highlight_lines(
        code,
        start=14,
        end=17,
        caption="You can also highlight multiple " "lines by ending the block with '# " "end'",
    )
    scene.highlight_none(code)
    scene.play(FadeOut(code))
    scene.clear()


def demo_render_self(scene: CodeScene):
    scene.add_background(f"{example_dir}/resources/blackboard.jpg")

    # Here is the code rendering this video you are watching now!
    code = scene.animate_code_comments(
        title="examples/intro.py",
        path=f"{example_dir}/intro.py",
        keep_comments=True,
        start_line=92,
        end_line=108,
        reset_at_end=False,
    )
    # end
    scene.wait(2)

    scene.play(FadeOut(code))
    scene.clear()


def demo_highlighting(scene: CodeScene):
    title = PangoText(
        """
        If you want more control, you can create code blocks and
        highlight them manually.
        """,
        font="Helvetica",
        line_spacing=0.5,
    ).scale(0.7)
    scene.play(ShowCreation(title, run_time=3, rate_func=linear))
    scene.wait(2)
    scene.clear()

    scene.add_background(f"{example_dir}/resources/blackboard.jpg")
    tex = scene.create_code(f"{example_dir}/highlights.py")
    scene.play(ShowCreation(tex))
    scene.highlight_line(
        tex,
        11,
        caption="Create code blocks yourself and pass in any arguments the Code class supports to do things "
        "like change the theme or font",
    )
    scene.highlight_lines(
        tex,
        13,
        19,
        caption="Highlight code with a caption to give extra information. A wait is"
        " automatically added for a time based on the length of the caption",
    )
    scene.highlight_line(tex, 21, caption="Reset highlighting and positioning")
    scene.highlight_none(tex)
    scene.play(FadeOut(tex))
    scene.clear()


def goodbye(scene: CodeScene):
    title = PangoText(
        """
        Try Code Video Generator today at:

          https://github.com/sleuth-io/code-video-generator

        Thanks for watching!""",
        font="Helvetica",
        line_spacing=0.5,
    ).scale(0.7)
    scene.play(ShowCreation(title, run_time=4, rate_func=linear))
    scene.wait(5)
    scene.play(FadeOut(title))


class Main(CodeScene):
    def construct(self):
        self.add_background_music(f"{example_dir}/resources/Pure Magic - Chris Haugen.mp3")
        title_scene(self)
        overview(self)
        demo_commenting(self)
        demo_highlighting(self)
        demo_render_self(self)
        goodbye(self)
